@@ -2,7 +2,7 @@
  * libdpkg - Debian packaging suite library routines
  * dpkg.h - general header for Debian package handling
  *
- * Copyright © 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright © 1994,1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2000,2001 Wichert Akkerman <wichert@debian.org>
  * Copyright © 2006-2015 Guillem Jover <guillem@debian.org>
  *
@@ -104,18 +104,16 @@ DPKG_BEGIN_DECLS
 #define DPKGSTAT	"dpkg-statoverride"
 #define DPKGTRIGGER	"dpkg-trigger"
 #define DPKG		"dpkg"
-#define DEBSIGVERIFY	"/usr/bin/debsig-verify"
+#define DEBSIGVERIFY	"debsig-verify"
 
 #define RM		"rm"
 #define CAT		"cat"
-#define FIND		"find"
 #define DIFF		"diff"
 
-#define FIND_EXPRSTARTCHARS "-(),!"
-
 #include <dpkg/progname.h>
 #include <dpkg/ehandle.h>
 #include <dpkg/report.h>
+#include <dpkg/string.h>
 #include <dpkg/program.h>
 
 /*** log.c ***/
