@@ -24,18 +24,17 @@ use warnings;
 
 our $VERSION = '0.02';
 
-use parent qw(Dpkg::Source::Package);
-
 use Cwd qw(abs_path getcwd);
 use File::Basename;
 use File::Temp qw(tempdir);
 
-use Dpkg;
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
 use Dpkg::Exit qw(push_exit_handler pop_exit_handler);
 use Dpkg::Source::Functions qw(erasedir);
 
+use parent qw(Dpkg::Source::Package);
+
 our $CURRENT_MINOR_VERSION = '0';
 
 # Remove variables from the environment that might cause git to do
@@ -52,26 +51,46 @@ sub import {
             return 1;
         }
     }
-    error(_g('cannot unpack git-format source package because ' .
+    error(g_('cannot unpack git-format source package because ' .
              'git is not in the PATH'));
 }
 
-sub sanity_check {
+sub _sanity_check {
     my $srcdir = shift;
 
     if (! -d "$srcdir/.git") {
-        error(_g('source directory is not the top directory of a git ' .
+        error(g_('source directory is not the top directory of a git ' .
                  'repository (%s/.git not present), but Format git was ' .
                  'specified'), $srcdir);
     }
     if (-s "$srcdir/.gitmodules") {
-        error(_g('git repository %s uses submodules; this is not yet supported'),
+        error(g_('git repository %s uses submodules; this is not yet supported'),
               $srcdir);
     }
 
     return 1;
 }
 
+my @module_cmdline = (
+    {
+        name => '--git-ref=<ref>',
+        help => N_('specify a git <ref> to include in the git bundle'),
+        when => 'build',
+    }, {
+        name => '--git-depth=<number>',
+        help => N_('create a shallow clone with <number> depth'),
+        when => 'build',
+    }
+);
+
+sub describe_cmdline_options {
+    my $self = shift;
+
+    my @cmdline = ( $self->SUPER::describe_cmdline_options(), @module_cmdline );
+
+    return @cmdline;
+}
+
 sub parse_cmdline_option {
     my ($self, $opt) = @_;
     return 1 if $self->SUPER::parse_cmdline_option($opt);
@@ -88,7 +107,7 @@ sub parse_cmdline_option {
 sub can_build {
     my ($self, $dir) = @_;
 
-    return (0, _g("doesn't contain a git repository")) unless -d "$dir/.git";
+    return (0, g_("doesn't contain a git repository")) unless -d "$dir/.git";
     return 1;
 }
 
@@ -100,17 +119,17 @@ sub do_build {
     my ($dirname, $updir) = fileparse($dir);
     my $basenamerev = $self->get_basename(1);
 
-    sanity_check($dir);
+    _sanity_check($dir);
 
     my $old_cwd = getcwd();
-    chdir($dir) or syserr(_g("unable to chdir to `%s'"), $dir);
+    chdir $dir or syserr(g_("unable to chdir to '%s'"), $dir);
 
     # Check for uncommitted files.
     # To support dpkg-source -i, get a list of files
     # equivalent to the ones git status finds, and remove any
     # ignored files from it.
     my @ignores = '--exclude-per-directory=.gitignore';
-    my $core_excludesfile = `git config --get core.excludesfile`;
+    my $core_excludesfile = qx(git config --get core.excludesfile);
     chomp $core_excludesfile;
     if (length $core_excludesfile && -e $core_excludesfile) {
         push @ignores, "--exclude-from=$core_excludesfile";
@@ -132,9 +151,9 @@ sub do_build {
           }
       }
     }
-    close($git_ls_files_fh) or syserr(_g('git ls-files exited nonzero'));
+    close($git_ls_files_fh) or syserr(g_('git ls-files exited nonzero'));
     if (@files) {
-        error(_g('uncommitted, not-ignored changes in working directory: %s'),
+        error(g_('uncommitted, not-ignored changes in working directory: %s'),
               join(' ', @files));
     }
 
@@ -143,19 +162,19 @@ sub do_build {
     my $tmp;
     my $shallowfile;
     if ($self->{options}{git_depth}) {
-        chdir($old_cwd) or syserr(_g("unable to chdir to `%s'"), $old_cwd);
+        chdir $old_cwd or syserr(g_("unable to chdir to '%s'"), $old_cwd);
         $tmp = tempdir("$dirname.git.XXXXXX", DIR => $updir);
         push_exit_handler(sub { erasedir($tmp) });
         my $clone_dir = "$tmp/repo.git";
         # file:// is needed to avoid local cloning, which does not
         # create a shallow clone.
-        info(_g('creating shallow clone with depth %s'),
+        info(g_('creating shallow clone with depth %s'),
                 $self->{options}{git_depth});
         system('git', 'clone', '--depth=' . $self->{options}{git_depth},
                '--quiet', '--bare', 'file://' . abs_path($dir), $clone_dir);
         subprocerr('git clone') if $?;
         chdir($clone_dir)
-            or syserr(_g("unable to chdir to `%s'"), $clone_dir);
+            or syserr(g_("unable to chdir to '%s'"), $clone_dir);
         $shallowfile = "$basenamerev.gitshallow";
         system('cp', '-f', 'shallow', "$old_cwd/$shallowfile");
         subprocerr('cp shallow') if $?;
@@ -165,7 +184,7 @@ sub do_build {
     my $bundlefile = "$basenamerev.git";
     my @bundle_arg=$self->{options}{git_ref} ?
         (@{$self->{options}{git_ref}}) : '--all';
-    info(_g('bundling: %s'), join(' ', @bundle_arg));
+    info(g_('bundling: %s'), join(' ', @bundle_arg));
     system('git', 'bundle', 'create', "$old_cwd/$bundlefile",
            @bundle_arg,
            'HEAD', # ensure HEAD is included no matter what
@@ -173,7 +192,7 @@ sub do_build {
     );
     subprocerr('git bundle') if $?;
 
-    chdir($old_cwd) or syserr(_g("unable to chdir to `%s'"), $old_cwd);
+    chdir $old_cwd or syserr(g_("unable to chdir to '%s'"), $old_cwd);
 
     if (defined $tmp) {
         erasedir($tmp);
@@ -200,38 +219,42 @@ sub do_extract {
             if (! defined $bundle) {
                 $bundle = $file;
             } else {
-                error(_g('format v3.0 (git) uses only one .git file'));
+                error(g_('format v3.0 (git) uses only one .git file'));
             }
         } elsif ($file =~ /^\Q$basenamerev\E\.gitshallow$/) {
             if (! defined $shallow) {
                 $shallow = $file;
             } else {
-                error(_g('format v3.0 (git) uses only one .gitshallow file'));
+                error(g_('format v3.0 (git) uses only one .gitshallow file'));
             }
         } else {
-            error(_g('format v3.0 (git) unknown file: %s', $file));
+            error(g_('format v3.0 (git) unknown file: %s', $file));
         }
     }
     if (! defined $bundle) {
-        error(_g('format v3.0 (git) expected %s'), "$basenamerev.git");
+        error(g_('format v3.0 (git) expected %s'), "$basenamerev.git");
     }
 
-    erasedir($newdirectory);
+    if ($self->{options}{no_overwrite_dir} and -e $newdirectory) {
+        error(g_('unpack target exists: %s'), $newdirectory);
+    } else {
+        erasedir($newdirectory);
+    }
 
     # Extract git bundle.
-    info(_g('cloning %s'), $bundle);
+    info(g_('cloning %s'), $bundle);
     system('git', 'clone', '--quiet', $dscdir . $bundle, $newdirectory);
     subprocerr('git bundle') if $?;
 
     if (defined $shallow) {
         # Move shallow info file into place, so git does not
         # try to follow parents of shallow refs.
-        info(_g('setting up shallow clone'));
+        info(g_('setting up shallow clone'));
         system('cp', '-f',  $dscdir . $shallow, "$newdirectory/.git/shallow");
         subprocerr('cp') if $?;
     }
 
-    sanity_check($newdirectory);
+    _sanity_check($newdirectory);
 }
 
 1;
